from netlogger.analysis.schema.stampede_schema import initializeToPegasusDB, func, orm, \
    Workflow as WorkflowTable, Workflowstate as WorkflowstateTable, Job as JobTable, \
    Jobstate as JobstateTable, Host as HostTable, Task as TaskTable, Edge as EdgeTable, \
    EdgeStatic as EdgeStaticTable
from netlogger.analysis.modules._base import SQLAlchemyInit
from netlogger.analysis.workflow.sql_alchemy import Workflow as WorkflowBase, \
    Job as JobBase, Host as HostBase, Task as TaskBase, Jobstate as JobstateBase, \
    Discovery as DiscoveryBase, Workflowstate as WorkflowstateBase

import pygraph

class Stats(DiscoveryBase):
    def __init__(self, connString):
        """
        Initialization method.  The manditory argument connectionInfo
        will be defined as appropriate in a subclass (string, dict, etc),
        and connect to the appropriate back end.
        """
        DiscoveryBase.__init__(self, connString)
        pass


    def wf_count(self):
        query = self.session.query(WorkflowTable.wf_uuid).group_by(WorkflowTable.wf_id).count()
        return query
    
    def job_count(self):
        query = self.session.query(JobTable).count()
        return query

    def task_count(self):
        query = self.session.query(TaskTable).count()
        return query

    def edge_count(self):
        query = self.session.query(EdgeTable).count()
        return query

    def get_all_counts(self):
		r = {}
		r['wfs'] = self.wf_count()
		r['jobs'] = self.job_count()
		r['tasks'] = self.task_count()
		r['edges'] = self.edge_count()
		return r
        

class Workflow(WorkflowBase):
    def __init__(self, connString=None):
        WorkflowBase.__init__(self, connString)
        
    def initialize(self, wf_id):
        WorkflowBase.initialize(self, wf_id)
        self.graph = pygraph.digraph()
        self.graph.wf_id = wf_id
        
    def build_graph(self):
        # retrieve edges from edge table
        query = self.session.query(EdgeStaticTable).filter(EdgeStaticTable.wf_uuid==self._wf_uuid)
        for e in query.all():
            if self.graph.has_node(e.parent)== False:
                self.graph.add_node(e.parent)
            if self.graph.has_node(e.child) ==False:
                self.graph.add_node(e.child)
            if self.graph.has_edge(e.parent, e.child) == False:
                self.graph.add_edge(e.parent, e.child)
        return self.graph
    
def main():
    import sys
    
    db_conn = sys.argv[1]
    
    d = Stats(db_conn)

    print d.get_all_counts()

    
    uuid = d.fetch_all()[0]
    w = Workflow(db_conn)
    w.initialize(uuid)
    graph = w.build_graph()

    dot = pygraph.readwrite.dot.write(graph)
    f = open(sys.argv[2], 'w')
    f.write(dot)
    f.close()
    
    cmd = 'dot ' + sys.argv[2] +  "> " + sys.argv[2] +"x"
    import os
    os.system(cmd)
    
    f = open(sys.argv[2] +"x", 'r')
    xdot = f.read()
    f.close()
    print xdot
    
    
    gr = pygraph.readwrite.dot.read(xdot)
    print gr.nodes
    
    sorted_list = pygraph.algorithms.sorting.topological_sorting(gr)
    print sorted_list
    
        #print w

if __name__ == '__main__':
    main()
